import os
from aws_pcluster_helpers.models.sinfo import SInfoTable, SinfoRow
from aws_pcluster_helpers.models import nextflow
from rich.console import Console
from rich.table import Table
import sys

from aws_pcluster_helpers.utils.logging import setup_logger

logger = setup_logger('nxf')


def main(outfile, overwrite, stdout):
    nxf_slurm_config = nextflow.NXFSlurmConfig()
    data = nxf_slurm_config.print_slurm_config()
    if outfile:
        if os.path.exists(outfile) and not overwrite:
            logger.warn(
                f'Outfile: {outfile} exists. Please specify --overwrite print your config or select a file that does not exist.')
            sys.exit(1)
        with open(outfile, 'w') as fh:
            fh.write(data)
    if stdout:
        print(data)
    else:
        logger.warn(f'Neither --outfile or --stdout were selected. Printing to the screen anyways.')
        print(data)



from __future__ import annotations
import functools
import os

from jinja2 import Environment, BaseLoader
from pydantic.dataclasses import dataclass
import dataclasses
from typing import List, Any, TypedDict, Dict, Optional
from typing import ForwardRef
from devtools import PrettyFormat, pprint, pformat, debug
import json
import yaml

from aws_pcluster_helpers import (PClusterConfig, InstanceTypesData, PClusterInstanceTypes, InstanceTypesMappings,
                                  size_in_gib)

from aws_pcluster_helpers.models.instance_types_data import (PClusterInstanceTypes, InstanceTypesMappings)

from aws_pcluster_helpers.models import sinfo
from aws_pcluster_helpers import (PClusterConfig, InstanceTypesData, PClusterInstanceTypes, InstanceTypesMappings,
                                  size_in_gib)
from aws_pcluster_helpers.utils.logging import setup_logger
from aws_pcluster_helpers.models.config import PClusterConfigFiles


# NXFProcess = ForwardRef('NXFProcess')


@dataclasses.dataclass
class NXFProcess(sinfo.SinfoRow):
    mem_min: int = 1
    cpu_min: int = 1


defaults = {
    'queue': None,
    'sinfo_name': None,
    'constraint': None,
    'ec2_instance_type': None,
    'gpus': None,
    'gpu': None,
    'extra': None
}


@dataclass
class NXFSlurmConfig(sinfo.SInfoTable):

    @functools.cached_property
    def processes(self) -> Dict[str, NXFProcess]:
        t_processes = {}
        for row in self.rows:
            row_data = row.__dict__
            label = row_data['label']
            t_processes[label] = NXFProcess(**row_data)
        return t_processes

    @functools.cached_property
    def default_processes(self) -> Dict[str, NXFProcess]:
        return {
            'tiny': NXFProcess(label='tiny',
                               mem_min=1, mem=6, cpu=1, **defaults),
            'low': NXFProcess(label='low', mem_min=2, cpu_min=1, mem=12, cpu=2, **defaults),
            'medium': NXFProcess(label='medium', mem_min=12, cpu_min=1, mem=36, cpu=6, **defaults),
            'high': NXFProcess(label='high', mem_min=36, cpu_min=6, mem=72, cpu=12, **defaults),
            'high_mem': NXFProcess(label='high_memory', mem_min=72, cpu_min=12, mem=200, cpu=12,
                                   **defaults),
            'long': NXFProcess(label='long', mem_min=12, cpu_min=1, mem=36, cpu=6, **defaults),
        }

    def map_nxf_default_labels_to_pcluster_queues(self):
        for key in self.default_processes.keys():
            # this doesn't take cpu into account...
            best_match = min(list(self.processes.keys()),
                             key=lambda x: abs(self.processes[x].mem - self.default_processes[key].mem))
            self.default_processes[key].queue = self.processes[best_match].queue
            self.default_processes[key].constraint = self.processes[best_match].constraint

    def __post_init__(self):
        self.map_nxf_default_labels_to_pcluster_queues()
        return

    def print_slurm_config(self):
        config_template_file = os.path.join(os.path.dirname(__file__), '_templates', 'slurm.config')
        if not os.path.exists(config_template_file):
            raise ValueError(f'Config template does not exist')
        with open(config_template_file, 'r+') as f:
            config_template = f.read()
        rtemplate = Environment(loader=BaseLoader).from_string(config_template)
        data = {
            'processes': self.processes,
            'default_processes': self.default_processes
        }
        return rtemplate.render(**data)



# generated by datamodel-codegen:
#   filename:  instance-types-data.json
#   timestamp: 2022-05-24T09:17:47+00:00

from __future__ import annotations
from datasize import DataSize
from copy import deepcopy
from typing import Dict

from mypy_boto3_ec2.type_defs import (InstanceTypeDef)
from pydantic.dataclasses import dataclass

from devtools import PrettyFormat, pprint, pformat, debug
import json
import os
import yaml


class Config:
    arbitrary_types_allowed = True


@dataclass(config=Config)
class InstanceTypesMappings:
    ec2_instance_types: Dict
    sinfo_instance_types: Dict

    @classmethod
    def from_json(cls, json_file) -> InstanceTypesMappings:
        """
        Data looks like this:
        "basic": {
        "t32xlarge": "t3.2xlarge",
        "t3alarge": "t3a.large",
        "t3axlarge": "t3a.xlarge"
        },
        A dictionary of queue name and sinfo instance type mapped back to the ec2 instance name
        """
        json_data = json.load(open(json_file))
        inverse_data = {}
        data = {}
        for queue in json_data.keys():
            instance_types = json_data[queue]
            for sinfo_instance_type in instance_types:
                t_inverse_data = {}
                t_data = {}
                instance_type = instance_types[sinfo_instance_type]
                t_inverse_data['ec2_instance_type'] = instance_type
                t_inverse_data['sinfo_instance_type'] = sinfo_instance_type
                if instance_type not in inverse_data.keys():
                    inverse_data[instance_type] = {}
                if sinfo_instance_type not in data.keys():
                    data[sinfo_instance_type] = {}
                inverse_data[instance_type].update(t_inverse_data)
                data[sinfo_instance_type].update(t_inverse_data)
                if 'queues' not in inverse_data[instance_type].keys():
                    inverse_data[instance_type]['queues'] = [queue]
                    data[sinfo_instance_type]['queues'] = [queue]
                else:
                    inverse_data[instance_type]['queues'].append(queue)
                    data[sinfo_instance_type]['queues'].append(queue)
        return InstanceTypesMappings(ec2_instance_types=inverse_data, sinfo_instance_types=data)


def size_in_gib(mib: int) -> int:
    mib_bytes = DataSize(f'{mib}Mi')
    return mib_bytes / mib_bytes.IEC_prefixes['Gi']


@dataclass(config=Config)
class InstanceTypesData:
    data: Dict

    @property
    def instance_type_data(self) -> InstanceTypeDef:
        if 'Hypervisor' in self.data.keys():
            if 'nitro' in self.data['Hypervisor']:
                self.data['Hypervisor'] = 'xen'
        if 'MemoryInfo' in self.data.keys():
            if 'SizeInMib' in self.data['MemoryInfo'].keys():
                size_in_mib = self.data['MemoryInfo']['SizeInMib']
                bytes = DataSize(f'{size_in_mib}Mi')
                gibs = bytes / bytes.IEC_prefixes['Gi']
                self.data['MemoryInfo']['SizeInGib'] = gibs
        else:
            debug(self.data)
        return deepcopy(self.data)

    @classmethod
    def from_yaml(cls, yaml_file: str) -> InstanceTypesData:
        d = yaml.safe_load(open(yaml_file))
        return InstanceTypesData(data=d)


@dataclass
class PClusterInstanceTypes:
    instance_type_data: Dict[str, InstanceTypesData]

    @classmethod
    def from_json(cls, json_file: str) -> PClusterInstanceTypes:
        data = json.load(open(json_file))
        instance_type_defs = {}
        for k in data.keys():
            instance_type_defs[k] = InstanceTypesData(data={'data': data[k]})

        return PClusterInstanceTypes(instance_type_data=instance_type_defs)



#!/usr/bin/env python

"""Tests for `aws_pcluster_helpers` package."""

import unittest

from aws_pcluster_helpers import (PClusterConfig, PClusterConfigFiles, InstanceTypesData, PClusterInstanceTypes,
                                  InstanceTypesMappings,
                                  size_in_gib)
from aws_pcluster_helpers.utils.logging import setup_logger
from aws_pcluster_helpers.models.config import (ENV_PCLUSTER_CONFIG_FILE, ENV_INSTANCE_TYPES_DATA_FILE,
                                                ENV_INSTANCE_TYPE_MAPPINGS_FILE)
import yaml
import json
import os
from devtools import PrettyFormat, pprint, pformat, debug

instance_types_data_file = os.path.join(os.path.dirname(__file__), 'instance-types-data.json')
instance_type_mapping_file = os.path.join(os.path.dirname(__file__), 'instance_name_type_mappings.json')
pcluster_config_file = os.path.join(os.path.dirname(__file__), 'pcluster_config.yml')
os.environ[ENV_INSTANCE_TYPE_MAPPINGS_FILE] = instance_type_mapping_file
os.environ[ENV_INSTANCE_TYPES_DATA_FILE] = instance_types_data_file
os.environ[ENV_PCLUSTER_CONFIG_FILE] = pcluster_config_file

logger = setup_logger(logger_name='tests', log_level='DEBUG')


def test_load_pcluster_config():
    pcluster_config = PClusterConfig.from_yaml(pcluster_config_file)
    assert pcluster_config


def test_load_instance_types_data():
    pcluster_instance_types = PClusterInstanceTypes.from_json(instance_types_data_file)
    debug(pcluster_instance_types)
    assert pcluster_instance_types


def test_load_instance_types_mapping():
    instance_types_mappings = InstanceTypesMappings.from_json(instance_type_mapping_file)
    assert instance_types_mappings


def test_nxf_gen():
    instance_types_mappings = InstanceTypesMappings.from_json(instance_type_mapping_file)
    pcluster_instance_types = PClusterInstanceTypes.from_json(instance_types_data_file)
    pcluster_config = PClusterConfig.from_yaml(pcluster_config_file)
    return


def test_pcluster_config_files():
    pcluster_config_files = PClusterConfigFiles()



import os
from aws_pcluster_helpers.models.sinfo import SInfoTable, SinfoRow
from aws_pcluster_helpers.models import nextflow
from rich.console import Console
from rich.table import Table


def main():
    sinfo = SInfoTable()
    table = sinfo.get_table()
    console = Console()
    console.print(table)



"""Top-level package for AWS PCluster Helpers."""

__author__ = """Jillian Rowe"""
__email__ = 'jillian@dabbleofdevops.com'
__version__ = '0.1.0'

from aws_pcluster_helpers.models.config import (PClusterConfigFiles)
from aws_pcluster_helpers.models.pcluster_config import (PClusterConfig, ComputeSettings, ComputeResource,
                                                         SlurmQueue)
from aws_pcluster_helpers.models.instance_types_data import (InstanceTypesData, PClusterInstanceTypes,
                                                             InstanceTypesMappings, size_in_gib)



from __future__ import annotations
import os
from dataclasses import dataclass
from typing import Any, Union
from pathlib import Path
from typing import List
from pydantic import BaseModel, dataclasses, fields
import yaml
import json
import os
import dataclasses
from typing import Union, Dict, Any, TypedDict
from pathlib import Path

ENV_PCLUSTER_CONFIG_FILE = 'PCLUSTER_CONFIG_FILE'
ENV_INSTANCE_TYPE_MAPPINGS_FILE = 'INSTANCE_TYPE_MAPPINGS_FILE'
ENV_INSTANCE_TYPES_DATA_FILE = 'INSTANCE_TYPES_DATA_FILE'


@dataclasses.dataclass
class PClusterConfigFiles:
    @property
    def pcluster_config_file(self) -> Union[str, Path]:
        return os.environ.get(ENV_PCLUSTER_CONFIG_FILE,
                              '/opt/parallelcluster/shared/cluster-config.yaml')

    @property
    def instance_name_type_mappings_file(self) -> Union[str, Path]:
        return os.environ.get(ENV_INSTANCE_TYPE_MAPPINGS_FILE,
                              '/opt/slurm/etc/pcluster/instance_name_type_mappings.json')

    @property
    def instance_types_data_file(self) -> Union[str, Path]:
        return os.environ.get(ENV_INSTANCE_TYPES_DATA_FILE,
                              '/opt/parallelcluster/shared/instance-types-data.json')



from __future__ import annotations

import pandas as pd
import functools
import os
from dataclasses import dataclass
from typing import Any, Union
from pathlib import Path
from typing import List, Optional, Union, Any
from pydantic import BaseModel, dataclasses, fields, Field
import yaml
import json
import os
import dataclasses
from typing import Union, Dict, Any, TypedDict
from pathlib import Path

from aws_pcluster_helpers import (PClusterConfig, InstanceTypesData, PClusterInstanceTypes, InstanceTypesMappings,
                                  size_in_gib)
from aws_pcluster_helpers.utils.logging import setup_logger
from aws_pcluster_helpers.models.config import PClusterConfigFiles
from rich.console import Console
from rich.table import Table

logger = setup_logger(logger_name='sinfo')


@dataclasses.dataclass
class SinfoRow:
    sinfo_name: str
    label: str
    queue: str
    constraint: str
    ec2_instance_type: str
    mem: int
    cpu: int
    gpu: Optional[dict]
    gpus: Optional[List]
    extra: Optional[dict]


# TODO add custom ami lookup

@dataclass
class SInfoTable:

    @functools.cached_property
    def table_columns(self) -> List[Any]:
        return [
            {'label': 'Queue', 'key': 'queue'},
            {'label': 'Constraint', 'key': 'constraint'},
            {'label': 'Mem(Gib)', 'key': 'mem'},
            {'label': 'CPU', 'key': 'cpu'},
            {'label': 'EC2', 'key': 'ec2_instance_type'},
        ]

    @functools.cached_property
    def pcluster_config_files(self) -> PClusterConfigFiles:
        return PClusterConfigFiles()

    @functools.cached_property
    def instance_type_mappings(self) -> InstanceTypesMappings:
        return InstanceTypesMappings.from_json(
            self.pcluster_config_files.instance_name_type_mappings_file)

    @functools.cached_property
    def pcluster_instance_types(self) -> PClusterInstanceTypes:
        return PClusterInstanceTypes.from_json(self.pcluster_config_files.instance_types_data_file)

    @functools.cached_property
    def pcluster_config(self) -> PClusterConfig:
        return PClusterConfig.from_yaml(self.pcluster_config_files.pcluster_config_file)

    @functools.cached_property
    def rows(self) -> List[SinfoRow]:
        instance_types_mappings = self.instance_type_mappings
        pcluster_instance_types = self.pcluster_instance_types

        sinfo_records = []
        for slurm_queue in self.pcluster_config.Scheduling.SlurmQueues:
            compute_resources = slurm_queue.ComputeResources
            queue = slurm_queue.Name
            for compute_resource in compute_resources:
                sinfo_label = {}
                sinfo_instance_type = compute_resource.Name
                ec2_instance_type = instance_types_mappings.sinfo_instance_types[sinfo_instance_type][
                    'ec2_instance_type']
                pcluster_instance_type = pcluster_instance_types.instance_type_data.get(ec2_instance_type)
                mem_in_mib = pcluster_instance_type.data['data']['MemoryInfo']['SizeInMiB']
                mem_in_gib = size_in_gib(mem_in_mib)
                cpu = pcluster_instance_type.data['data']['VCpuInfo']['DefaultCores']
                gpu = None
                gpus = []
                if 'GpuInfo' in pcluster_instance_type.data['data']:
                    gpu = pcluster_instance_type.data['data']['GpuInfo']
                    gpu_total_mem_in_mib = gpu["TotalGpuMemoryInMiB"]
                    gpu_total_mem_in_gib = size_in_gib(gpu_total_mem_in_mib)
                    gpus.append(gpu)
                cpu = pcluster_instance_type.data['data']['VCpuInfo']['DefaultCores']
                p_type = 'dy'
                if compute_resource.MinCount > 0:
                    p_type = 'st'
                label = f"{slurm_queue.Name.replace('-', '_')}_{p_type}__{ec2_instance_type.replace('.', '_')}"
                sinfo_name = f"{slurm_queue.Name}-{p_type}-{sinfo_instance_type}"

                sinfo_label['name'] = label
                sinfo_label['max_mem'] = int(mem_in_gib)
                sinfo_label['max_cpu'] = cpu
                sinfo_label['queue'] = queue
                sinfo_label['constraint'] = sinfo_instance_type
                sinfo_records.append(
                    SinfoRow(
                        mem=int(mem_in_gib),
                        cpu=cpu,
                        sinfo_name=sinfo_name,
                        queue=queue,
                        constraint=sinfo_instance_type,
                        ec2_instance_type=ec2_instance_type,
                        label=label,
                        gpus=gpus,
                        extra={},
                        gpu=None,
                    )
                )
        return sinfo_records

    @functools.cached_property
    def dataframe(self) -> pd.DataFrame:
        records = []
        for record in self.rows:
            records.append(record.__dict__)
        df = pd.DataFrame.from_records(records)
        df.sort_values(by=['queue', 'mem', 'cpu'], inplace=True)
        return df

    def get_table(self):
        if not len(self.rows):
            logger.error(f'SLURM SInfo - none found... exiting')
            raise ValueError(f'SLURM SInfo - none found ... exiting')

        table = Table(title="SLURM SInfo", show_lines=True)
        colors = ["turquoise4", "purple4", "deep_sky_blue3", "hot_pink3", "dodger_blue1"]
        queue_colors = {}
        for column in self.table_columns:
            table.add_column(column['label'], justify="right", no_wrap=True)
        table.add_column('SBATCH', justify="left")
        for row in self.dataframe.to_records():
            t_row = []
            if row['queue'] not in queue_colors:
                queue_colors[row['queue']] = colors.pop()
            queue = row['queue']
            constraint = row['constraint']
            cpus = row['cpu']
            color = queue_colors[row['queue']]
            for t in self.table_columns:
                key = t['key']
                value = row[key]
                t_row.append(f"[{color}]{str(value)}")
            t_row.append(
                f"[{color}]#SBATCH --partition={queue}\n#SBATCH --constraint={constraint}\n#SBATCH -c={cpus}")
            table.add_row(*t_row)

        return table



import rich_click as click
from rich_click import RichCommand, RichGroup

from aws_pcluster_helpers.commands import cli_sinfo
from aws_pcluster_helpers.commands import cli_gen_nxf_slurm_config


@click.group()
def cli():
    """
    Helper functions for aws parallelcluster.
    """
    return


@cli.command()
def sinfo():
    """
    A more helpful sinfo
    """
    click.echo("Printing sinfo table")
    cli_sinfo.main()


@cli.command()
@click.option("--output", "-o", help="Output path", required=False)
@click.option("--overwrite", is_flag=True, help="Overwrite local files")
@click.option("--stdout", is_flag=True, help="Write slurm config to stdout")
def gen_nxf_slurm_config(output: str, overwrite: bool, stdout: bool):
    """
    Generate a slurm.config for nextflow that is compatible with your cluster.

    You will see a process label for each partition/node type.

    Use the configuration in your process by setting the label to match the label in the config.
    """
    click.echo("Generating NXF Slurm config")
    cli_gen_nxf_slurm_config.main(output, overwrite, stdout)


if __name__ == "__main__":
    cli()



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'devtools',
    'colorlog',
    'pydantic',
    'boto3',
    'boto3-stubs[essential]',
    'datasize',
    'rich-click',
    'pandas',
    'cookiecutter'
]

test_requirements = []

setup(
    author="Jillian Rowe",
    author_email='jillian@dabbleofdevops.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Helpers to generate different configurations for PCluster",
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='aws_pcluster_helpers',
    name='aws_pcluster_helpers',
    entry_points={
        'console_scripts': [
            'pcluster-helper=aws_pcluster_helpers.cli:cli',
        ],
    },
    packages=find_packages(include=['aws_pcluster_helpers', 'aws_pcluster_helpers.*']),
    data_files=[
        (
            'templates', [
                'aws_pcluster_helpers/models/_templates/slurm.config',
            ]
        ),
    ],
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/dabble-of-devops-bioanalyze/aws_pcluster_helpers',
    version='0.2.0',
    zip_safe=False,
)



# generated by datamodel-codegen:
#   filename:  pcluster_config.yml
#   timestamp: 2022-05-24T09:17:46+00:00

from __future__ import annotations

from typing import List

from pydantic import BaseModel
import yaml
import json
import os
from dataclasses import dataclass
from typing import Union, Dict, Any, TypedDict
from pathlib import Path


class Image(BaseModel):
    Os: str
    CustomAmi: str


class Tag(BaseModel):
    Key: str
    Value: str


class Networking(BaseModel):
    ElasticIp: bool
    SubnetId: str
    AdditionalSecurityGroups: List[str]


class Dcv(BaseModel):
    Enabled: bool
    Port: int


class Ssh(BaseModel):
    KeyName: str


class RootVolume(BaseModel):
    Size: int


class EphemeralVolume(BaseModel):
    MountDir: str


class LocalStorage(BaseModel):
    RootVolume: RootVolume
    EphemeralVolume: EphemeralVolume


class S3Acces(BaseModel):
    BucketName: str
    EnableWriteAccess: bool


class Iam(BaseModel):
    S3Access: List[S3Acces]


class OnNodeConfigured(BaseModel):
    Script: str


class CustomActions(BaseModel):
    OnNodeConfigured: OnNodeConfigured


class HeadNode(BaseModel):
    InstanceType: str
    Networking: Networking
    Dcv: Dcv
    Ssh: Ssh
    LocalStorage: LocalStorage
    Iam: Iam
    CustomActions: CustomActions


class CloudWatch(BaseModel):
    Enabled: bool
    RetentionInDays: int


class Logs(BaseModel):
    CloudWatch: CloudWatch


class CloudWatch1(BaseModel):
    Enabled: bool


class Dashboards(BaseModel):
    CloudWatch: CloudWatch1


class Monitoring(BaseModel):
    Logs: Logs
    Dashboards: Dashboards


class FsxLustreSettings(BaseModel):
    StorageCapacity: int
    DeploymentType: str
    DataCompressionType: str


class SharedStorageItem(BaseModel):
    MountDir: str
    Name: str
    StorageType: str
    FsxLustreSettings: FsxLustreSettings


class RootVolume1(BaseModel):
    Size: int


class EphemeralVolume1(BaseModel):
    MountDir: str


class LocalStorage1(BaseModel):
    RootVolume: RootVolume1
    EphemeralVolume: EphemeralVolume1


class ComputeSettings(BaseModel):
    LocalStorage: LocalStorage1


class ComputeResource(BaseModel):
    Name: str
    InstanceType: str
    MinCount: int = 0
    MaxCount: int = 1


class OnNodeConfigured1(BaseModel):
    Script: str


class CustomActions1(BaseModel):
    OnNodeConfigured: OnNodeConfigured1


class S3Acces1(BaseModel):
    BucketName: str
    EnableWriteAccess: bool


class Iam1(BaseModel):
    S3Access: List[S3Acces1]


class Image1(BaseModel):
    CustomAmi: str


class Networking1(BaseModel):
    SubnetIds: List[str]
    AdditionalSecurityGroups: List[str]


class SlurmQueue(BaseModel):
    Name: str
    ComputeSettings: ComputeSettings
    ComputeResources: List[ComputeResource]
    CustomActions: CustomActions1
    Iam: Iam1
    Image: Image1
    Networking: Networking1


class Scheduling(BaseModel):
    Scheduler: str
    SlurmQueues: List[SlurmQueue]


class PClusterConfig(BaseModel):
    Region: str
    Image: Image
    Tags: List[Tag]
    HeadNode: HeadNode
    Monitoring: Monitoring
    SharedStorage: List[SharedStorageItem]
    Scheduling: Scheduling

    @classmethod
    def from_yaml(cls, yaml_file: str) -> PClusterConfig:
        data = yaml.safe_load(open(yaml_file).read())
        return PClusterConfig(**data)
